unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, VCLTee.TeEngine,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.TeeTools, VCLTee.Series,
  Vcl.StdCtrls, cxGraphics, cxLookAndFeels, cxLookAndFeelPainters, Vcl.Menus,
  cxButtons, System.Generics.Collections;
{$define CreateBar}

type
  TChartColorBands = TObjectList<TColorBandTool>;

  TForm1 = class(TForm)
    Chart1: TChart;
    cbHighlight: TColorBandTool;
    Panel1: TPanel;
    chkUseCalloutPos: TCheckBox;
    Panel2: TPanel;
    lbxEvents: TListBox;
    Panel3: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lblMouseDown: TLabel;
    lblMouseMove: TLabel;
    lblMouseUp: TLabel;
    btnClearTimes: TButton;
    Label4: TLabel;
    lblStartIdx: TLabel;
    Label6: TLabel;
    lblEndIdx: TLabel;
    Timer1: TTimer;
    Label5: TLabel;
    lblChartMousePos: TLabel;
    Series1: TBarSeries;
    cxButton1: TcxButton;
    procedure btnClearTimesClick(Sender: TObject);
    procedure Chart1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure Chart1MouseUp(Sender: TObject; Button: TMouseButton; Shift:
        TShiftState; X, Y: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure cxButton1Click(Sender: TObject);
  private
    FOffset: double;
    FBands: TChartColorBands;
    procedure AddTimeBand;
    function GetXPosFromValue(const aXValue: double): integer;
    function GetLastBandXValue: integer;
    function GetValueIndex(const aXPos: integer): integer;
    procedure CheckListBox;
    procedure ClearMouseTimes;
    { Private declarations }
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  Form1: TForm1;

implementation


{$R *.dfm}

function TForm1.GetLastBandXValue: integer;
begin
  result := 1;
end;

function TForm1.GetValueIndex(const aXPos: integer): integer;
var I,
  lDistance: integer;

  procedure UpdateMinDistance(const aValueIndex: Integer);
  var
    lXPos: integer;
    lCalcedDist: integer;
  begin
    lXPos:= Series1.CalcXPos(aValueIndex);
    lXPos:= lXPos + (Series1.BarWidth div 2);
    lCalcedDist := abs(aXPos - lXPos);

    if (lDistance < 0) or (lDistance > lCalcedDist) then
    begin
      lDistance:= lCalcedDist;
      result := aValueIndex;
    end;
  end;

begin
  Assert(Series1 is TBarSeries);

  result := - 1;

  if (Series1.Count > 0) then
  begin
    lDistance := -1;
    UpdateMinDistance(0);
    for I := 1 to Series1.Count-1 do
      UpdateMinDistance(I);
  end;
end;

function TForm1.GetXPosFromValue(const aXValue: double): integer;
begin
  result :=  Chart1.BottomAxis.CalcXPosValue(aXValue);
end;

procedure TForm1.Chart1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  lIdx: Integer;
begin
  lblMouseDown.Caption := TimeToStr(Now);
  Caption := 'Down: ' + TimeToStr(Now);
  lIdx := GetValueIndex(X);
  if lIdx > -1 then
  begin
    cbHighlight.StartValue := lIdx + FOffset;
    cbHighlight.EndValue := lIdx + FOffset;
    cbHighlight.Visible := True;
  end
  else
    cbHighlight.Visible := false;

  if cbHighlight.Visible then
  begin
    lbxEvents.Items.Insert(0, 'MouseDown');
    CheckListBox;
  end;

end;

constructor TForm1.Create(aOwner: TComponent);
var
  I: Integer;
  lValue: double;
begin
  inherited;

  FBands := TChartColorBands.Create;

  FOffset := 0;

  Chart1.View3D:=false;


  {$if defined(CreateBar)}
  Chart1.FreeAllSeries;
  Series1 := TBarSeries.Create(self);
  Series1.Marks.Visible := false;
  Chart1.AddSeries(Series1);
  {$endif}

  lValue := 1000;
  for I := 1 to 25 do
  begin
    Series1.AddXY(I, lValue);
    lValue := lValue * 1.03;
  end;



  cbHighlight.Visible := false;

  cbHighlight.StartLine.AllowDrag := false;
  cbHighlight.EndLine.AllowDrag := false;
  cbHighlight.AllowDrag := false;

  ClearMouseTimes;

end;

procedure TForm1.cxButton1Click(Sender: TObject);
begin
  AddTimeBand;
end;

destructor TForm1.Destroy;
begin
  FBands.Free;
  inherited;
end;

procedure TForm1.AddTimeBand;
var
  lIdx: Integer;
  lPt: TPoint;
begin
  lIdx := GetValueIndex(GetLastBandXValue);
  if lIdx > -1 then
  begin
    lPt.X := GetXPosFromValue(GetLastBandXValue);
    lPt.Y := Chart1.LeftAxis.CalcYPosValue(Chart1.LeftAxis.Maximum * 0.5);
    lPt := Chart1.ClientToScreen(lPt);

    SetCursorPos(lPt.X + 1, lPt.Y);
    cbHighlight.Visible := True;
  end
  else
    cbHighlight.Visible := false;

  if cbHighlight.Visible then
  begin
    lbxEvents.Items.Insert(0, 'MouseDown');
    CheckListBox;
  end;

end;

procedure TForm1.btnClearTimesClick(Sender: TObject);
begin
  ClearMouseTimes;
end;

procedure TForm1.ClearMouseTimes;
begin
  lblMouseDown.Caption := '-';
  lblMouseMove.Caption := '-';
  lblMouseUp.Caption := '-';
end;
procedure TForm1.Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y:
    Integer);
var
  lIdx: Integer;
  lPt: TPoint;
  lText: string;
begin
  lblChartMousePos.Caption := Format('%d, %d', [X, Y]);

  if cbHighlight.Visible then
  begin
    lblMouseMove.Caption := TimeToStr(Now);
    lText := Format('MouseMove: X: %d; Y: %d', [X, Y]);
    if (lbxEvents.Items.Count > 0) and
      (lbxEvents.Items.Objects[0] = TObject(1)) then
      lbxEvents.Items[0] := lText
    else
      lbxEvents.Items.InsertObject(0, 'MouseMove', TObject(1));
    CheckListBox;

    lIdx := GetValueIndex(X);
    if lIdx > -1 then
      cbHighlight.EndValue := lIdx + FOffset;
  end;
end;

procedure TForm1.Chart1MouseUp(Sender: TObject; Button: TMouseButton; Shift:
    TShiftState; X, Y: Integer);
begin
if cbHighlight.Visible then
begin
  lblMouseUp.Caption := TimeToStr(Now);
  lbxEvents.Items.Insert(0, 'MouseUp');
end;
  cbHighlight.Visible := false;

//  cxHintStyleController1.HideHint;
end;

procedure TForm1.CheckListBox;
begin
  while lbxEvents.Items.Count > 20 do
    lbxEvents.Items.Delete(lbxEvents.Items.Count-1);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  if cbHighlight.Visible then
  begin
    lblStartIdx.Caption := IntToStr(Trunc(cbHighlight.StartValue));
    lblEndIdx.Caption := IntToStr(Trunc(cbHighlight.EndValue));
  end
  else
  begin
    lblStartIdx.Caption := '-';
    lblEndIdx.Caption := '-';
  end;
end;

end.
